#import "GraphView.h"
#import "BiorhythmAppDelegate.h"
#import "BiorhythmViewController.h"
#import "UICalloutView.h"

@implementation GraphView

- (void)awakeFromNib {
	/* 배경 이미지로 사용할 bio.png 로드 */
	backgroundImage = [[UIImage alloc] initWithContentsOfFile:
					   [[NSBundle mainBundle] pathForResource:@"bio" ofType:@"png"]];
}

- (void)drawRect:(CGRect)rect {
	BiorhythmAppDelegate* appDelegate = 
								(BiorhythmAppDelegate*)
									[[UIApplication sharedApplication] delegate];
	BiorhythmViewController* viewController = [appDelegate viewController];
	double (*biorhythmData)[MAX_BIORHYTHMDATATYPE];
	biorhythmData = viewController->_biorhythmData;
	NSInteger currDay = viewController->currDay;
	
	/* 데이터별 색상 테이블 */
	static const CGFloat color[MAX_BIORHYTHMDATATYPE][4] = {
		{ 0.0, 1.0, 0.0, 1.0 }, // 신체
		{ 1.0, 0.2, 1.0, 1.0 }, // 감성
		{ 1.0, 0.6, 0.0, 1.0 }  // 지성
	};
	
	CGSize viewSize = rect.size;
	CGContextRef context = UIGraphicsGetCurrentContext();
	
	/* 배경 이미지 출력 */
	CGPoint point;
	point.x = point.y = 0;
//	[backgroundImage drawInRect:rect];
	[backgroundImage drawAtPoint:point];
	
	/* 오늘 날짜를 가르키는 위치는 빨간 선으로 출력 */
	CGContextBeginPath(context);
	CGContextSetLineWidth(context, 1.0);
	CGContextSetRGBStrokeColor(context, 1.0, 0.0, 0.0, 1.0);
	CGContextAddRect(context, CGRectMake(START_X + (DAY_WIDTH * (currDay - 1)), START_Y, 
										 2, viewSize.height - BOTTOM_HEIGHT));

	CGContextStrokePath(context);
	
	/* 바이오리듬 출력 */
	int i, j;
	int lineX = START_X;
	
	int vcenter = START_Y + (viewSize.height - START_Y - BOTTOM_HEIGHT)/2; // 그래프의 Y 중간 좌표
	
	/* 바이오리듬 출력 */
	for (i = 1; i < 31; i++) {
		for(j = 0; j < MAX_BIORHYTHMDATATYPE; j++) {
			CGContextBeginPath(context);
			CGContextSetLineWidth(context, 2.0);
			
			CGContextSetStrokeColor(context, color[j]);
			
			CGContextMoveToPoint(context, lineX, vcenter - biorhythmData[i-1][j]);
			CGContextAddLineToPoint(context, lineX + DAY_WIDTH, vcenter - biorhythmData[i][j]);
			CGContextStrokePath(context);
			
			/* 아이콘 출력 */
			if (currDay == i) {
				CGPoint center = CGPointMake(lineX,  vcenter - biorhythmData[i-1][j]);
				if (j == 0)
					imgPhysical.center = center;
				else if (j == 1)
					imgSensitivity.center = center;
				else
					imgIntellectual.center = center;
			}
		}
		lineX += DAY_WIDTH;
	}
}

-(void) touchesBegan:(NSSet*)touches withEvent:(UIEvent*) event
{
	CGPoint tp = [[touches anyObject] locationInView:self];
	CGPoint anchorPoint;
	UICalloutView* callout = [[UICalloutView alloc] init];
	
	BiorhythmAppDelegate* appDelegate = 
					(BiorhythmAppDelegate*) [[UIApplication sharedApplication] delegate];
	BiorhythmViewController* viewController = [appDelegate viewController];
	double (*biorhythmData)[MAX_BIORHYTHMDATATYPE];
	biorhythmData = viewController->_biorhythmData;
	/* 터치 포인트와 가장 가까운 바이오리듬 좌표 계산 */
	CGSize viewSize = [self frame].size;
	int vcenter = START_Y + (viewSize.height - START_Y - BOTTOM_HEIGHT)/2; // 그래프의 Y 중간 좌표
	int lineX = START_X;
	double minDist = 480;
	NSArray* type = [NSArray arrayWithObjects:@"신체",@"감성",@"지성", nil];
	for (int i = 0; i < 31; i++) {
		for(int j = 0; j < MAX_BIORHYTHMDATATYPE; j++) {
			CGPoint cp = CGPointMake(lineX, vcenter - biorhythmData[i][j]);
			double dist = sqrt((cp.x-tp.x)*(cp.x-tp.x) + 
							   (cp.y-tp.y)*(cp.y-tp.y));
			if (dist < minDist) {
				minDist = dist;
				anchorPoint = cp;
				[callout setTitle:
					[NSString stringWithFormat:
						@"%@[%d]", [type objectAtIndex:j],
						(int) biorhythmData[i][j]]];
			}
		}
		
		lineX += DAY_WIDTH;
	}
	
	[callout addTarget:self action:@selector(closeDisclosure:)];
	[self addSubview:callout];
	[callout setAnchorPoint:anchorPoint 
			   boundaryRect:CGRectMake(0, 0, 320, 100) 
					animate:YES];
}
-(IBAction) closeDisclosure:(id)calloutButton
{
	UICalloutView* callout = (UICalloutView*) [calloutButton superview];
	[callout fadeOutWithDuration:0.5f];
	[callout removeFromSuperview];
}

- (void)dealloc {
	[backgroundImage release];
	[super dealloc];
}

@end
