//
//  MyUIView.m
//  FingerDraw
//
//  Created by 유동근 on 09. 04. 22.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "MyUIView.h"

#define BrushSize              6.0
#define BrushAlpha             0.5

@implementation MyUIView

- (id)initWithFrame:(CGRect)frame {
	CGContextRef            context;
	CGColorSpaceRef         colorSpace;
	
	if((self = [super initWithFrame:frame])) {
		//Create the offscreen context that contains our finger-painted image i.e. "painting context"
		colorSpace = CGColorSpaceCreateDeviceRGB();
		_cgContext = CGBitmapContextCreate(NULL, frame.size.width, frame.size.height, 8, 4 * frame.size.width, colorSpace, kCGImageAlphaPremultipliedFirst);
		CGColorSpaceRelease(colorSpace);
		if(_cgContext == NULL) {
			[self release];
			return nil;
		}
		
		//Create another offscreen context to contain the current finger stroke i.e. "stroke context" (we need to use a separate context so that blending of the current stroke over the painting context is done right)
		_cgLayer = CGLayerCreateWithContext(_cgContext, frame.size, NULL);
		context = CGLayerGetContext(_cgLayer);
		CGContextSetLineWidth(context, BrushSize);
		CGContextSetLineCap(context, kCGLineCapRound);
		CGContextSetRGBStrokeColor(context, 1.0, 0.0, 0.0, 1.0);
	}
	
	return self;
}

- (void)drawRect:(CGRect)rect {
	[self.superview drawRect:rect];
	
	CGRect			bounds = [self bounds];
	CGContextRef	context = UIGraphicsGetCurrentContext();
	CGImageRef		image;
	       
	//Copy the contents of the painting context to the view's context
	image = CGBitmapContextCreateImage(_cgContext);
	CGContextDrawImage(context, bounds, image);
	CGImageRelease(image);
	
	//Blend on top the contents of the stroke context
	CGContextSetAlpha(context, BrushAlpha);
	CGContextDrawLayerInRect(context, bounds, _cgLayer);
}

-(void) touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	UIScrollView* scrollView = (UIScrollView*) self.superview.superview;
	BOOL bScrollEnable = (2 <= [[event allTouches] count]);
	if (!bScrollEnable) 
		[scrollView setScrollEnabled:NO];
	else
		return;

	UITouch*	touch = [touches anyObject];
	
	//If the user double-taps, clear the whole painting
	if (2 <= [touch tapCount]) {
		CGContextClearRect(_cgContext, [self bounds]);
		[self setNeedsDisplay];
	}
	//Otherwise, draw a line segment in the stroke context
	else
		[self touchesMoved:touches withEvent:event];
}

-(void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	UITouch*	touch = [touches anyObject];
	CGPoint		location = [touch locationInView:self],
				lastLocation = [touch previousLocationInView:self];
	CGContextRef	context = CGLayerGetContext(_cgLayer);
	
	//Draw a line segment in the stroke context
	CGContextBeginPath(context);
	CGContextMoveToPoint(context, lastLocation.x, lastLocation.y);
	CGContextAddLineToPoint(context, location.x, location.y);
	CGContextStrokePath(context);
	      
	//We need to redraw the view
	[self setNeedsDisplay];
}

- (void) touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event
{
	UIScrollView* scrollView = (UIScrollView*) self.superview.superview;
	[scrollView setScrollEnabled:YES];
	
	CGRect	bounds = [self bounds];
	      
	//Merge the content of the stroke context with the painting context
	CGContextSetAlpha(_cgContext, BrushAlpha);
	CGContextDrawLayerInRect(_cgContext, bounds, _cgLayer);
	       
	//Clear the stroke context
	CGContextClearRect(CGLayerGetContext(_cgLayer), bounds);
	       
	//We need to redraw the view
	[self setNeedsDisplay];
}

-(BOOL) isMultipleTouchEnabled
{
	return YES;
}

- (void)dealloc {
    [super dealloc];
	CGLayerRelease(_cgLayer);
	CGContextRelease(_cgContext);
}


@end
